@@ -4,7 +4,7 @@ use warnings;
 use strict;
 use 5.10.1;
 
-our $VERSION = '1.776';
+our $VERSION = '1.779';
 
 use Exporter qw( import );
 
@@ -18,9 +18,9 @@ our @EXPORT_OK = qw( table_char_width );
 
 sub table_char_width { [
 #[     0x0,     0x1f, -1],
-[    0x20,     0x7e, 1],
+#[    0x20,     0x7e, 1],
 #[    0x7f,     0x9f, -1],
-[    0xa0,    0x2ff, 1],
+#[    0xa0,    0x2ff, 1],
 [   0x300,    0x36f, 0],
 [   0x370,    0x482, 1],
 [   0x483,    0x489, 0],
@@ -34,7 +34,9 @@ sub table_char_width { [
 [   0x5c4,    0x5c5, 0],
 [   0x5c6,    0x5c6, 1],
 [   0x5c7,    0x5c7, 0],
-[   0x5c8,    0x60f, 1],
+[   0x5c8,    0x5ff, 1],
+[   0x600,    0x605, 0],
+[   0x606,    0x60f, 1],
 [   0x610,    0x61a, 0],
 [   0x61b,    0x61b, 1],
 [   0x61c,    0x61c, 0],
@@ -43,8 +45,8 @@ sub table_char_width { [
 [   0x660,    0x66f, 1],
 [   0x670,    0x670, 0],
 [   0x671,    0x6d5, 1],
-[   0x6d6,    0x6dc, 0],
-[   0x6dd,    0x6de, 1],
+[   0x6d6,    0x6dd, 0],
+[   0x6de,    0x6de, 1],
 [   0x6df,    0x6e4, 0],
 [   0x6e5,    0x6e6, 1],
 [   0x6e7,    0x6e8, 0],
@@ -72,12 +74,12 @@ sub table_char_width { [
 [   0x829,    0x82d, 0],
 [   0x82e,    0x858, 1],
 [   0x859,    0x85b, 0],
-[   0x85c,    0x896, 1],
+[   0x85c,    0x88f, 1],
+[   0x890,    0x891, 0],
+[   0x892,    0x896, 1],
 [   0x897,    0x89f, 0],
 [   0x8a0,    0x8c9, 1],
-[   0x8ca,    0x8e1, 0],
-[   0x8e2,    0x8e2, 1],
-[   0x8e3,    0x902, 0],
+[   0x8ca,    0x902, 0],
 [   0x903,    0x939, 1],
 [   0x93a,    0x93a, 0],
 [   0x93b,    0x93b, 1],
@@ -1014,7 +1016,7 @@ sub table_char_width { [
 [ 0xe0020,  0xe007f, 0],
 [ 0xe0080,  0xe00ff, 1],
 [ 0xe0100,  0xe01ef, 0],
-[ 0xe01f0, 0x10ffff, 1],
+#[ 0xe01f0, 0x10ffff, 1],
 ] }
 
 
